clc; clear;

% Anonymous function for Black Scholes call
BSC = @(s,K,r,q,v,T) s*exp(-q*T)*normcdf((log(s/K) + (r-q+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r-q-v^2/2)*T)/v/sqrt(T));

% Input the spot, risk free rate, dividend, maturity, volatility
S0 = 30;
r = 0.06;
q = 0.02;
T = 0.25;
v = 0.2;

% Forward price
F = S0*exp((r-q)*T);

% Strikes
K = 20:.05:45;
NK = length(K);

% Mean and variance of lognormal distr under Black-Scholes
mu = log(S0) + (r-q-v^2/2)*T;
sigma = v*sqrt(T);

% Closed form RND for Black-Scholes is lognormal
for k=1:NK
    BSRND(k) = 1/(K(k)*sigma*sqrt(2*pi)) * exp(-0.5*(log(K(k))-mu)^2/sigma^2);
end

% Generate call prices
for k=1:NK
    CallPrice(k) = BSC(S0,K(k),r,q,v,T);
end

% Extract the discounted RND using the Breeden and Litzenberger formula
type = 'cubic';
[RND,K2] = ExtractRND(K,CallPrice,type);

% The Breeden-Litzenberger RND
RND = RND.*exp(r*T);

% Plot both RNDs
plot(K,BSRND,'k',K2,RND,'r')
line([F F],[0 max(max(RND),max(BSRND))])
legend('ClosedForm','Breeden','Forward Price')

